import fedmsg
import fedmsg.encoding
import fedmsg.consumers

import json
import requests

from . import config
from .jobtrigger import JobTrigger
from . import exceptions as exc


def _pull_version_release(compose_id):
    """Splits out the relevant bits of the compose ID. Input requires the form
    "Fedora-TYPE-VERSION-Date.Serial" (ie, Fedora-Cloud-24-20170224.0) that fedmsg
    spits out in compose_id."""

    parts = compose_id.split('-')
    if len(parts) < 4:
        raise(exc.TriggerMsgError("Provided compose_id did not contain"
                                  "all parts."))
    else:
        name = '-'.join(parts[:2])
        version = parts[1]
        release = parts[2]

        return name, version, release


def _fetch_metadata(compose_url):
    """This method simply isolates the one network call for the metadata out
    to make it easier to test the other logic. We'll presume that the 'requests'
    library does what it's supposed to.

    Input is the 'location' from fedmsg and it returns a json object from the
    metadata file."""

    metadata_path = "/metadata/images.json"
    metadata = requests.get(compose_url + metadata_path)
    if metadata.status_code is not 200:
        raise(exc.TriggerMsgError("Couldn't pull metadata from Koji."))

    metadata = json.loads(metadata.text)

    return metadata


def _find_image(compose_url):
    """Parse out the images from the compose metadata to determine which test to
    to run. This does not include Rawhide composes, as those include more than
    one qcow2 image to test."""

    metadata = _fetch_metadata(compose_url)

    x86_images = metadata['payload']['images']['CloudImages']['x86_64']

    for image in x86_images:
        if image['format'] == 'qcow2':
            # Have to add the / here to make a valid URL because the compose in
            # the fedmsg doesn't have a trailing slash and the path in the
            # metadata doesn't have a leading slash.
            return compose_url + "/" + image['path']


def _process(msg):
    """This method preps the incoming fedmsg data so the trigger can set
    off the task. Expects fedmsg.body data as input and returns a dict
    for JobTrigger.do_trigger."""

    MESSAGE_TYPE = "CloudCompose"

    if msg['msg']['status'] != "FINISHED":
        raise(exc.TriggerMsgError("Rejecting message because status not "
                                  "'FINISHED'."))

    item = _find_image(msg['msg']['location'])

    name, version, release = _pull_version_release(msg['msg']['compose_id'])

    if version == "Atomic":
        MESSAGE_TYPE = "AtomicCompose"

    data = {
        "_msg": msg,
        "message_type": MESSAGE_TYPE,
        "item": item,
        "item_type": 'compose',
        "name": name,
        "version": version,
        "release": release,
    }

    return data


class CloudComposeJobTrigger(JobTrigger):

    def process(self, msg):
        return _process(msg)


class CloudComposeCompletedConsumer(fedmsg.consumers.FedmsgConsumer):
    jsonify = False
    topic = "org.fedoraproject.{}.pungi.compose.status.change".format(config.deployment_type)
    config_key = "taskotron.cloudcomposecompletedconsumer.enabled"

    def __init__(self, *args, **kw):
        super(CloudComposeCompletedConsumer, self).__init__(*args, **kw)
        self.trigger = CloudComposeJobTrigger(self.log)

    def consume(self, message):
        msg = fedmsg.encoding.loads(message.body)
        try:
            data = self.trigger.process(msg)
            self.trigger.do_trigger(data)
        except exc.TriggerMsgError as e:
            self.log.debug(e)
            return
